@@ -1,4 +1,18 @@
 # -*- cperl -*-
+# Copyright (C) 2004-2006 MySQL AB
+# 
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 2 of the License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 # This is a library file used by the Perl version of mysql-test-run,
 # and is part of the translation of the Bourne shell script with the
@@ -222,8 +236,10 @@ sub mtr_report_stats ($) {
       "the documentation at\n",
       "http://www.mysql.com/doc/en/MySQL_test_suite.html\n";
   }
-  print
-      "The servers were restarted $tot_restarts times\n";
+  if (!$::opt_extern)
+  {
+    print "The servers were restarted $tot_restarts times\n";
+  }
 
   if ( $::opt_timer )
   {
@@ -279,6 +295,18 @@ sub mtr_report_stats ($) {
           }
         }
       }
+
+      if ( $::opt_check_testcases )
+      {
+        # Look for warnings produced by mysqltest in testname.warnings
+        foreach my $test_warning_file
+	  ( glob("$::glob_mysql_test_dir/r/*.warnings") )
+        {
+          $found_problems= 1;
+	  print WARN "Check myqltest warnings in $test_warning_file\n";
+        }
+      }
+
       if ( $found_problems )
       {
 	mtr_warning("Got errors/warnings while running tests, please examine",
@@ -329,7 +357,7 @@ sub mtr_report_stats ($) {
 
   if ( $tot_failed != 0 || $found_problems)
   {
-    mtr_error("there where failing test cases");
+    mtr_error("there were failing test cases");
   }
 }
 
